/*=============================================================================
    Copyright (c) 2002-2003 Joel de Guzman
    Copyright (c) 2002 Juan Carlos Arevalo-Baeza
    Copyright (c) 2002-2003 Martin Wille
    http://spirit.sourceforge.net/

  Distributed under the Boost Software License, Version 1.0. (See accompanying
  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
=============================================================================*/
#ifndef BOOST_SPIRIT_DYNAMIC_HPP
#define BOOST_SPIRIT_DYNAMIC_HPP

#include <boost/spirit/home/classic/version.hpp>

///////////////////////////////////////////////////////////////////////////////
//
//  Master header for Spirit.Dynamic
//
///////////////////////////////////////////////////////////////////////////////
#include <boost/spirit/home/classic/dynamic/if.hpp>
#include <boost/spirit/home/classic/dynamic/for.hpp>
#include <boost/spirit/home/classic/dynamic/while.hpp>
#include <boost/spirit/home/classic/dynamic/lazy.hpp>
#include <boost/spirit/home/classic/dynamic/stored_rule.hpp>
#include <boost/spirit/home/classic/dynamic/rule_alias.hpp>
#include <boost/spirit/home/classic/dynamic/select.hpp>
#include <boost/spirit/home/classic/dynamic/switch.hpp>

////////////////////////////////////////////////////////////////////////////////
#endif // BOOST_SPIRIT_DYNAMIC_HPP
