
#ifndef BOOST_CONTRACT_DETAIL_INLINED_HPP_
#define BOOST_CONTRACT_DETAIL_INLINED_HPP_

// Copyright (C) 2008-2018 Lorenzo Caminiti
// Distributed under the Boost Software License, Version 1.0 (see accompanying
// file LICENSE_1_0.txt or a copy at http://www.boost.org/LICENSE_1_0.txt).
// See: http://www.boost.org/doc/libs/release/libs/contract/doc/html/index.html

#include <boost/contract/detail/inlined/old.hpp>
#include <boost/contract/detail/inlined/core/exception.hpp>
#include <boost/contract/detail/inlined/detail/checking.hpp>

#endif // #include guard

